@@ -1,6 +1,6 @@
 /*
  * mpstat: per-processor statistics
- * (C) 2000-2004 by Sebastien GODARD (sysstat <at> wanadoo.fr)
+ * (C) 2000-2005 by Sebastien GODARD (sysstat <at> wanadoo.fr)
  *
  ***************************************************************************
  * This program is free software; you can redistribute it and/or modify it *
@@ -15,7 +15,7 @@
  *                                                                         *
  * You should have received a copy of the GNU General Public License along *
  * with this program; if not, write to the Free Software Foundation, Inc., *
- * 675 Mass Ave, Cambridge, MA 02139, USA.                                 *
+ * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA                   *
  ***************************************************************************
  */
 
@@ -50,8 +50,7 @@ unsigned char *cpu_bitmap;	/* Bit 0: Global; Bit 1: 1st proc; etc. */
 struct mp_timestamp st_mp_tstamp[DIM];
 /* Nb of processors on the machine. A value of 1 means two processors */
 int cpu_nr = -1;
-long interval = 0, count = 0;
-unsigned int flags = 0;
+long interval = -1, count = 0;
 
 
 /*
@@ -140,7 +139,7 @@ void write_stats_core(short prev, short curr, short dis,
     * that may happen since uptime values are unsigned long long but are
     * calculated as a sum of values that _may_ be unsigned long only...
     */
-   if (WANT_BOOT_STATS(flags))
+   if (!interval)
       itv = st_mp_tstamp[curr].uptime;
    else
       itv = (st_mp_tstamp[curr].uptime - st_mp_tstamp[prev].uptime) & 0xffffffff;
@@ -150,27 +149,25 @@ void write_stats_core(short prev, short curr, short dis,
 
    /* Print stats */
    if (dis)
-      printf(_("\n%-11s  CPU   %%user   %%nice %%system %%iowait    %%irq   %%soft   %%idle    intr/s\n"),
+      printf("\n%-11s  CPU   %%user   %%nice    %%sys %%iowait    %%irq   "
+	     "%%soft   %%idle    intr/s\n",
 	     prev_string);
 
    /* Check if we want global stats among all proc */
    if (*cpu_bitmap & 1) {
 
-      printf(_("%-11s  all"), curr_string);
+      printf("%-11s  all", curr_string);
 
-      printf("  %6.2f  %6.2f  %6.2f  %6.2f  %6.2f  %6.2f",
+      printf("  %6.2f  %6.2f  %6.2f  %6.2f  %6.2f  %6.2f  %6.2f",
 	     ll_sp_value(st_mp_cpu[prev]->cpu_user,    st_mp_cpu[curr]->cpu_user,    itv),
  	     ll_sp_value(st_mp_cpu[prev]->cpu_nice,    st_mp_cpu[curr]->cpu_nice,    itv),
 	     ll_sp_value(st_mp_cpu[prev]->cpu_system,  st_mp_cpu[curr]->cpu_system,  itv),
 	     ll_sp_value(st_mp_cpu[prev]->cpu_iowait,  st_mp_cpu[curr]->cpu_iowait,  itv),
 	     ll_sp_value(st_mp_cpu[prev]->cpu_hardirq, st_mp_cpu[curr]->cpu_hardirq, itv),
-	     ll_sp_value(st_mp_cpu[prev]->cpu_softirq, st_mp_cpu[curr]->cpu_softirq, itv));
-
-      if (st_mp_cpu[curr]->cpu_idle < st_mp_cpu[prev]->cpu_idle)
-	 printf("    %.2f", 0.0);	/* Handle buggy kernels */
-      else
-	 printf("  %6.2f",
-		ll_sp_value(st_mp_cpu[prev]->cpu_idle, st_mp_cpu[curr]->cpu_idle, itv));
+	     ll_sp_value(st_mp_cpu[prev]->cpu_softirq, st_mp_cpu[curr]->cpu_softirq, itv),
+	     (st_mp_cpu[curr]->cpu_idle < st_mp_cpu[prev]->cpu_idle) ?
+	     0.0 :	/* Handle buggy kernels */
+	     ll_sp_value(st_mp_cpu[prev]->cpu_idle, st_mp_cpu[curr]->cpu_idle, itv));
    }
 
    /*
@@ -178,7 +175,7 @@ void write_stats_core(short prev, short curr, short dis,
     * using the uptime computed for proc#0.
     */
    if (cpu_nr) {
-      if (WANT_BOOT_STATS(flags))
+      if (!interval)
 	 itv = st_mp_tstamp[curr].uptime0;
       else
 	 itv = (st_mp_tstamp[curr].uptime0 - st_mp_tstamp[prev].uptime0) & 0xffffffff;
@@ -202,21 +199,16 @@ void write_stats_core(short prev, short curr, short dis,
       st_mp_cpu_i = st_mp_cpu[curr] + cpu;
       st_mp_cpu_j = st_mp_cpu[prev] + cpu;
 
-      printf("  %6.2f  %6.2f  %6.2f  %6.2f  %6.2f  %6.2f",
+      printf("  %6.2f  %6.2f  %6.2f  %6.2f  %6.2f  %6.2f  %6.2f %9.2f\n",
 	     ll_sp_value(st_mp_cpu_j->cpu_user,    st_mp_cpu_i->cpu_user,    itv),
 	     ll_sp_value(st_mp_cpu_j->cpu_nice,    st_mp_cpu_i->cpu_nice,    itv),
 	     ll_sp_value(st_mp_cpu_j->cpu_system,  st_mp_cpu_i->cpu_system,  itv),
 	     ll_sp_value(st_mp_cpu_j->cpu_iowait,  st_mp_cpu_i->cpu_iowait,  itv),
 	     ll_sp_value(st_mp_cpu_j->cpu_hardirq, st_mp_cpu_i->cpu_hardirq, itv),
-	     ll_sp_value(st_mp_cpu_j->cpu_softirq, st_mp_cpu_i->cpu_softirq, itv));
-
-      if (st_mp_cpu_i->cpu_idle < st_mp_cpu_j->cpu_idle)
-	 printf("    %.2f", 0.0);
-      else
-	 printf("  %6.2f",
-		ll_sp_value(st_mp_cpu_j->cpu_idle, st_mp_cpu_i->cpu_idle, itv));
-
-      printf(" %9.2f\n",
+	     ll_sp_value(st_mp_cpu_j->cpu_softirq, st_mp_cpu_i->cpu_softirq, itv),
+	     (st_mp_cpu_i->cpu_idle < st_mp_cpu_j->cpu_idle) ?
+	     0.0 :
+	     ll_sp_value(st_mp_cpu_j->cpu_idle, st_mp_cpu_i->cpu_idle, itv),
 	     ll_s_value(st_mp_cpu_j->irq, st_mp_cpu_i->irq, itv));
    }
 }
@@ -267,27 +259,23 @@ void write_stats(short curr, short dis, struct tm *loc_time)
 /*
  ***************************************************************************
  * Read stats from /proc/stat
- * See kernel sources:
- * 2.4: linux/fs/proc/proc_misc.c: kstat_read_proc()
- * 2.6: linux/fs/proc/proc_misc.c: show_stat()
  ***************************************************************************
  */
 void read_proc_stat(short curr)
 {
-   FILE *statfp;
+   FILE *fp;
    struct mp_stats *st_mp_cpu_i;
    static char line[80];
    unsigned long long cc_user, cc_nice, cc_system, cc_hardirq, cc_softirq;
    unsigned long long cc_idle, cc_iowait;
    int proc_nb;
 
-   /* Open stat file */
-   if ((statfp = fopen(STAT, "r")) == NULL) {
+   if ((fp = fopen(STAT, "r")) == NULL) {
       fprintf(stderr, _("Cannot open %s: %s\n"), STAT, strerror(errno));
       exit(2);
    }
 
-   while (fgets(line, 80, statfp) != NULL) {
+   while (fgets(line, 80, fp) != NULL) {
 
       if (!strncmp(line, "cpu ", 4)) {
 	 /*
@@ -366,22 +354,18 @@ void read_proc_stat(short curr)
 	 sscanf(line + 5, "%llu", &(st_mp_cpu[curr]->irq));
    }
 
-   /* Close stat file */
-   fclose(statfp);
+   fclose(fp);
 }
 
 
 /*
  ***************************************************************************
  * Read stats from /proc/interrupts
- * See kernel sources:
- * 2.4: linux/arch/{i386,...}/kernel/irq.c: get_irq_list()
- * 2.6: linux/arch/{i386,...}/kernel/irq.c: show_interrupts()
  ***************************************************************************
  */
 void read_interrupts_stat(short curr)
 {
-   FILE *irqfp;
+   FILE *fp;
    struct mp_stats *st_mp_cpu_i;
    static char line[INTERRUPTS_LINE];
    unsigned long irq = 0;
@@ -392,10 +376,9 @@ void read_interrupts_stat(short curr)
       st_mp_cpu_i->irq = 0;
    }
 
-   /* Open interrupts file */
-   if ((irqfp = fopen(INTERRUPTS, "r")) != NULL) {
+   if ((fp = fopen(INTERRUPTS, "r")) != NULL) {
 
-      while (fgets(line, INTERRUPTS_LINE, irqfp) != NULL) {
+      while (fgets(line, INTERRUPTS_LINE, fp) != NULL) {
 
 	 if (isdigit(line[2])) {
 	
@@ -407,8 +390,7 @@ void read_interrupts_stat(short curr)
 	 }
       }
 
-      /* Close serial file */
-      fclose(irqfp);
+      fclose(fp);
    }
 }
 
@@ -424,6 +406,31 @@ void rw_mp_stat_loop(short dis_hdr, unsigned long lines, int rows,
 {
    short curr = 1, dis = 1;
 
+   st_mp_tstamp[0].hour   = loc_time->tm_hour;
+   st_mp_tstamp[0].minute = loc_time->tm_min;
+   st_mp_tstamp[0].second = loc_time->tm_sec;
+
+   /* Read stats */
+   read_proc_stat(0);
+   read_interrupts_stat(0);
+
+   if (!interval) {
+      /* Display since boot time */
+      st_mp_tstamp[1] = st_mp_tstamp[0];
+      memset(st_mp_cpu[1], 0, MP_STATS_SIZE * (cpu_nr + 2));
+      write_stats(0, DISP_HDR, loc_time);
+      exit(0);
+   }
+
+   /* Set a handler for SIGALRM */
+   alarm_handler(0);
+
+   /* Save the first stats collected. Will be used to compute the average */
+   st_mp_tstamp[2] = st_mp_tstamp[0];
+   memcpy(st_mp_cpu[2], st_mp_cpu[0], MP_STATS_SIZE * (cpu_nr + 2));
+
+   pause();
+
    do {
 
       /* Resetting the structure not needed since every fields will be set */
@@ -485,7 +492,7 @@ int main(int argc, char **argv)
 #endif
 
    /* How many processors on this machine ? */
-   get_cpu_nr(&cpu_nr, ~0);
+   cpu_nr = get_cpu_nr(~0);
 
    /*
     * cpu_nr: a value of 1 means there are 2 processors (0 and 1).
@@ -528,20 +535,18 @@ int main(int argc, char **argv)
 	    usage(argv[0]);
       }
 
-      else if (!interval) {		/* Get interval */
-	 if ((strspn(argv[opt], DIGITS) != strlen(argv[opt])) ||
-	     WANT_BOOT_STATS(flags))
+      else if (interval < 0) {		/* Get interval */
+	 if (strspn(argv[opt], DIGITS) != strlen(argv[opt]))
 	    usage(argv[0]);
 	 interval = atol(argv[opt]);
-	 if (!interval)
-	    flags |= F_BOOT_STATS;
-	 else if (interval < 0)
-	   usage(argv[0]);
+	 if (interval < 0)
+	    usage(argv[0]);
 	 count = -1;
       }
 
       else if (count <= 0) {		/* Get count value */
-	 if (strspn(argv[opt], DIGITS) != strlen(argv[opt]))
+	 if ((strspn(argv[opt], DIGITS) != strlen(argv[opt])) ||
+	     !interval)
 	    usage(argv[0]);
 	 count = atol(argv[opt]);
 	 if (count < 1)
@@ -562,9 +567,9 @@ int main(int argc, char **argv)
       rows = get_win_height();
       lines = rows;
    }
-   if (!interval)
+   if (interval < 0)
       /* Interval not set => display stats since boot time */
-      flags |= F_BOOT_STATS;
+      interval = 0;
 
    /* Get time */
    get_localtime(&loc_time);
@@ -573,33 +578,6 @@ int main(int argc, char **argv)
    uname(&header);
    print_gal_header(&loc_time, header.sysname, header.release, header.nodename);
 
-   st_mp_tstamp[0].hour   = loc_time.tm_hour;
-   st_mp_tstamp[0].minute = loc_time.tm_min;
-   st_mp_tstamp[0].second = loc_time.tm_sec;
-
-   /* Read stats */
-   read_proc_stat(0);
-   read_interrupts_stat(0);
-
-   if (WANT_BOOT_STATS(flags)) {
-      /* Display since boot time */
-      st_mp_tstamp[1] = st_mp_tstamp[0];
-
-      memset(st_mp_cpu[1], 0, MP_STATS_SIZE * (cpu_nr + 2));
-
-      write_stats(0, DISP_HDR, &loc_time);
-      exit(0);
-   }
-
-   /* Set a handler for SIGALRM */
-   alarm_handler(0);
-
-   /* Save the first stats collected. Will be used to compute the average */
-   st_mp_tstamp[2] = st_mp_tstamp[0];
-   memcpy(st_mp_cpu[2], st_mp_cpu[0], MP_STATS_SIZE * (cpu_nr + 2));
-
-   pause();
-
    /* Main loop */
    rw_mp_stat_loop(dis_hdr, lines, rows, &loc_time);
 
